import yaml
from collections import OrderedDict
from .compat import Path, text

_sorted = sorted


class namespace(OrderedDict):
    """
    A dictionary whose values can be accessed also as attributes
    and can be loaded and dumped as YAML.
    """

    def __init__(self, *args, **kwd):
        super(namespace, self).__init__(*args, **kwd)

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError as e:
            raise AttributeError(name)

    def __setattr__(self, name, value):
        if name.startswith("_"):
            super(namespace, self).__setattr__(name, value)
        else:
            self[name] = value

    def __delattr__(self, name):
        try:
            del self[name]
        except KeyError:
            super(namespace, self).__delattr__(name)

    def __getitem__(self, name):
        if not hasattr(name, "split"):
            return super(namespace, self).__getitem__(name)
        parts = name.split(".", 1)
        result = super(namespace, self).__getitem__(parts[0])
        if parts[1:]:
            return result[parts[1]]
        return result

    def __setitem__(self, name, value):
        if not hasattr(name, "split"):
            return super(namespace, self).__setitem__(name, value)
        parts = name.split(".", 1)
        if not parts[1:]:
            return super(namespace, self).__setitem__(parts[0], value)
        level = self.setdefault(parts[0], ns())
        level[parts[1]] = value

    def __dir__(self):

        def isidentifier(candidate):
            "Is the candidate string an identifier in Python 2.x"
            try:
                return candidate.isidentifier()
            except AttributeError:
                pass
            import keyword
            import re

            is_not_keyword = candidate not in keyword.kwlist
            pattern = re.compile(r"^[a-z_][a-z0-9_]*$", re.I)
            matches_pattern = bool(pattern.match(candidate))
            return is_not_keyword and matches_pattern

        try:
            attributes = super(namespace, self).__dir__()
        except AttributeError:  # Py2 OrderedDict has no __dir__
            attributes = []
        attributes.extend(k for k in self.keys() if isidentifier(k))
        return attributes

    def deepcopy(self):
        return self.loads(self.dump())

    @classmethod
    def deep(cls, x, sorted=False):
        """Turns recursively all the dicts of a json like
        structure into yamlns namespaces. Set sorted to true
        to force alphabetical order of the keys
        so that their dumps can be compared.
        """
        sort_function = _sorted if sorted else lambda x: x
        if type(x) in (dict, namespace):
            return ns(
                (k, cls.deep(v, sorted=sorted)) for k, v in sort_function(x.items())
            )
        if type(x) in (list, tuple):
            return [cls.deep(y, sorted=sorted) for y in x]
        return x

    @classmethod
    def loads(cls, yamlContent):
        yamlContent = text(yamlContent)
        import io

        return cls.load(io.StringIO(yamlContent))

    @classmethod
    def load(cls, source):
        from .serialization import NamespaceYAMLLoader

        # Already open read file
        if hasattr(source, "read"):
            return yaml.load(stream=source, Loader=NamespaceYAMLLoader)

        with Path(source).open() as f:
            return yaml.load(stream=f, Loader=NamespaceYAMLLoader)

    def dump(self, target=None):

        def dumpit(stream):
            from .serialization import NamespaceYamlDumper

            return yaml.dump(
                self,
                stream=stream,
                default_flow_style=False,
                allow_unicode=True,
                Dumper=NamespaceYamlDumper,
            )

        if target is None:
            return dumpit(target)

        # Already open write file
        if hasattr(target, "write"):
            return dumpit(target)

        import sys

        mode = "wb" if sys.version_info[0] == 2 else "w"

        with Path(target).open(mode) as f:
            return dumpit(f)

    @classmethod
    def fromTemplateVars(clss, templateContent):
        """Given a string with format template substitutions
        it builds a namespace having the fields to fill it.
        Namespace leaf values will be set as empty strings.
        """
        templateVariables = _collectVars(templateContent)
        return _varsTree(templateVariables)


def _collectVars(content):
    import re

    pattern = r"{([^}^[]*)(\[[^]]\])?}"
    return [item.group(1) for item in re.finditer(pattern, content)]


def _varsTree(theVars):
    ns = namespace()
    for segments in (var.split(".") for var in sorted(theVars)):
        target = ns
        for segment in segments[:-1]:
            if segment not in target:
                target[segment] = namespace()
            # TODO: double check it is a ns
            target = target[segment]
        target[segments[-1]] = ""
    return ns


ns = namespace  # alias

# vim: sw=4 ts=4 noet



import pytest

# This ensures that the file is registered for assert rewriting
# before assert_impl is imported

pytest.register_assert_rewrite("yamlns._pytestutils_asserts.assert_impl")
from .assert_impl import assert_ns_equal



from __future__ import unicode_literals
from .pytestutils import (
    yaml_snapshot,
    text_snapshot,
    test_name,
    assert_ns_equal,
    assert_ns_contains,
    normalize,
    ns,
)
from .compat import Path, py2
import pytest
import sys
import re


def rmrf(path):
    path = Path(path)
    if not path.exists():
        return
    if path.is_file():
        path.unlink()
        return
    for child in path.glob("*"):
        rmrf(child)
    path.rmdir()


ANSI_RE = re.compile(r"\x1b\[[0-9;]*m")


def strip_ansi(s):
    return ANSI_RE.sub("", s)


@pytest.fixture
def clean_snapshotdir():
    """Ensures the snapshotdir is removed before and after the test"""
    snapshotdir = Path("testdata/snapshots")
    rmrf(snapshotdir)
    yield snapshotdir
    rmrf(snapshotdir)


@pytest.fixture
def snapshotdir(clean_snapshotdir):
    """Ensures an empty existing snapshotdir, removed after the test"""
    clean_snapshotdir.mkdir(exist_ok=True, parents=True)
    yield clean_snapshotdir


def assertContent(path, expected):
    path = Path(path)
    assert path.exists()
    assert path.read_text(encoding="utf8") == expected


# normalize


def test__normalize__str():
    assert normalize("text") == "text"


def test__normalize__int():
    assert normalize(1) == 1


def test__normalize__float():
    x = normalize(1.2)
    assert x == 1.2
    assert type(x) == float


def test__normalize__list():
    x = [1, 2, 3]
    assert normalize(x) == [1, 2, 3]
    assert normalize(x) is not [1, 2, 3]  # a copy


def test__normalize__dict__turns_into_ns():
    x = dict()
    assert normalize(x) == dict()
    assert type(normalize(x)) == ns


def test__normalize__dict_inside_list__turns_into_ns():
    x = [dict()]
    assert normalize(x) == [dict()]
    assert type(normalize(x)[0]) == ns


def test__normalize__keys_get_sorted():
    x = normalize(
        ns.loads(
            """
        key2: value2
        key1: value1
    """
        )
    )
    assert x.dump() == ("key1: value1\n" "key2: value2\n")


# test_name fixture


def test__test_name(test_name):
    assert test_name == "yamlns.pytestutils_test.test__test_name"


# yaml_snapshot fixture


def test__yaml_snapshot__no_expectation(
    yaml_snapshot,
    clean_snapshotdir,
    test_name,
):
    with pytest.raises(AssertionError) as exception:
        yaml_snapshot("content")

    expected = (
        "First snapshot, check results and accept them with:\n"
        "mv testdata/snapshots/{0}.result testdata/snapshots/{0}.expected\n"
    ).format(test_name)
    assert format(exception.value)[: len(expected)] == expected
    assertContent(
        clean_snapshotdir / "{}.result".format(test_name), "snapshot: content\n"
    )
    assert not (clean_snapshotdir / "{}.expected".format(test_name)).exists()


def test__yaml_snapshot__differentExpectation(
    yaml_snapshot,
    snapshotdir,
    test_name,
):
    expectation = snapshotdir / (test_name + ".expected")
    expectation.write_text("snapshot: unexpected\n", encoding="utf8")

    with pytest.raises(AssertionError) as exception:
        yaml_snapshot("content")

    expected = (
        "Failed snapshot. Check the result and if it is ok accept it with:\n"
        "mv testdata/snapshots/{0}.result testdata/snapshots/{0}.expected\n"
    ).format(test_name)
    assert format(exception.value)[: len(expected)] == expected

    assertContent(snapshotdir / "{}.result".format(test_name), "snapshot: content\n")
    assertContent(
        snapshotdir / "{}.expected".format(test_name), "snapshot: unexpected\n"
    )


def test__yaml_snapshot__ok(yaml_snapshot, snapshotdir, test_name):
    expectation = snapshotdir / (test_name + ".expected")
    expectation.write_text("snapshot: content\n", encoding="utf8")

    yaml_snapshot("content")

    assert not (snapshotdir / "{}.result".format(test_name)).exists()
    assertContent(snapshotdir / "{}.expected".format(test_name), "snapshot: content\n")


# assert_ns_equal assertion


def test__assert_ns_equal__value_differs():
    with pytest.raises(AssertionError) as exception:
        assert_ns_equal("key: result", "key: expected")
    assert (
        "  - key: result\n" "  + key: expected"
        if py2
        else "  - key: expected\n" "  + key: result"
    ) in strip_ansi(format(exception.value))


def test__assert_ns_equal__result_dict():
    # Should not raise
    assert_ns_equal(dict(key="value"), "key: value")


def test__assert_ns_equal__expected_dict():
    # Should not raise
    assert_ns_equal("key: value", dict(key="value"))


def test__assert_ns_equal__inner_values_differs():
    with pytest.raises(AssertionError) as exception:
        assert_ns_equal(
            "parent:\n" "  key1: mybad\n" "  key2: value2\n",
            "parent:\n" "  key2: value2\n" "  key1: value1\n",
        )
    assert (
        "    parent:\n"
        "  -   key1: mybad\n"
        "  +   key1: value1\n"
        "      key2: value2"
        if py2
        else "    parent:\n"
        "  -   key1: value1\n"
        "  +   key1: mybad\n"
        "      key2: value2"
    ) in strip_ansi(format(exception.value))


def test__assert_ns_equal__bothYaml():
    # Should not raise
    assert_ns_equal("key: same", "key: same")


def test__assert_ns_equal__inverted_key_order():
    # Should not raise
    assert_ns_equal(
        "key1: value1\n" "key2: value2\n", "key2: value2\n" "key1: value1\n"
    )


def test__assert_ns_equal__inner_inverted_key_order():
    # Should not raise
    assert_ns_equal(
        "parent:\n" "  key1: value1\n" "  key2: value2\n",
        "parent:\n" "  key2: value2\n" "  key1: value1\n",
    )


def test__assert_ns_equal__common_indentation_ignored():
    # Should not raise
    assert_ns_equal(
        """
        parent:
          key1: value1
          key2: value2
    """,
        """
        parent:
          key1: value1
          key2: value2
    """,
    )


def test__assert_ns_equal__float_yaml():
    # Should not raise
    assert_ns_equal("1.2", "1.2")


def test__assert_ns_equal__string_yaml():
    # Should not raise
    assert_ns_equal("text", "text")


def test__assert_ns_contains__missing_key():
    with pytest.raises(AssertionError) as exception:
        assert_ns_contains(
            ns(common="common"),
            ns(common="common", missing="value"),
        )
    assert strip_ansi(format(exception.value)) == (
        "assert 'common: common\\n' == 'common: common\\nmissing: value\\n'\n"
        "    common: common\n"
        "  + missing: value"
        if py2
        else "assert 'common: common\\n' == 'common: comm...sing: value\\n'\n  \n"
        "    common: common\n"
        "  - missing: value"
    )


def test__assert_ns_contains__differing_key():
    with pytest.raises(AssertionError) as exception:
        assert_ns_contains(
            ns(common="different"),
            ns(common="expected"),
        )
    print(repr(format(exception.value)))
    assert strip_ansi(format(exception.value)) == (
        "assert 'common: different\\n' == 'common: expected\\n'\n"
        "  - common: different\n"
        "  + common: expected"
        if py2
        else "assert 'common: different\\n' == 'common: expected\\n'\n  \n"
        "  - common: expected\n"
        "  + common: different"
    )


def test__assert_ns_contains__key_missing_in_expectation_ignored():
    assert_ns_contains(
        ns(akey="value", ignored="value"),
        ns(akey="value"),
    )



#!/usr/bin/python3

from .core import ns
from .compat import text, Path


def apply(yamlfile, template, output, encoding="utf-8"):
    """
    Fills a template file with data in yamlfile to generate output.
    """
    data = ns.load(yamlfile)
    content = Path(template).read_text(encoding=encoding)
    result = content.format(**data)
    Path(output).write_text(text(result, encoding=encoding), encoding=encoding)


def extract(input_template, output_yaml, encoding="utf-8"):
    """
    Extracts a yaml file with the attribute structure deduced from template value insertions
    and empty values so you can edit them and provide the proper data.
    """
    content = Path(input_template).read_text(encoding=encoding)
    schema = ns.fromTemplateVars(content)
    Path(output_yaml).write_text(
        text(schema.dump(), encoding=encoding), encoding=encoding
    )


# vim: sw=4 ts=4 noet



#!/usr/bin/python3

from ..nstemplate import apply, extract
import sys


def main(args=sys.argv):  # pragma: no cover
    import argparse

    parser = argparse.ArgumentParser(
        description="Takes a template and extract a yaml to fill it.",
    )
    parser.add_argument(
        "--encoding",
        default=None,
        help="forces input encoder for templates",
        dest="encoding",
    )

    sub = parser.add_subparsers(
        dest="subcommand",
    )

    subparser = sub.add_parser(
        "extract",
        help="Collects the template vars from a template and renders an empty yaml file with them",
    )
    subparser.add_argument(
        metavar="input.template",
        dest="input_template",
    )
    subparser.add_argument(
        metavar="output.yaml",
        dest="output_yaml",
    )

    subparser = sub.add_parser(
        "apply",
        help="Apply structured data in a YAML file to a template",
    )
    subparser.add_argument(
        metavar="input.yaml",
        dest="input_yaml",
    )
    subparser.add_argument(
        metavar="template",
        dest="template",
    )
    subparser.add_argument(
        metavar="output",
        dest="output",
    )

    args = parser.parse_args()

    if args.subcommand == "extract":
        extract(args.input_template, args.output_yaml, args.encoding)
        return 0

    if args.subcommand == "apply":
        apply(args.input_yaml, args.template, args.output, args.encoding)
        return 0

    parser.print_help()
    return -1


def deprecated():  # pragma: no cover
    print(
        "The script nstemplate.py is deprecated. "
        "Use nstemplate instead (without the .py extension)"
    )
    sys.exit(-1)


if __name__ == "__main__":  # pragma: no cover
    sys.exit(main())


# vim: sw=4 ts=4 noet



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

import datetime
from .compat import text


def date(*adate):
    """Transforms almost anything into a datetime.date"""
    return Date(*adate)


def slashDate(adate):
    """Turns date into common Spanish 31/01/2015 format"""
    return Date(adate).slashDate


def isoDate(adate):
    """Turns a date in almost any format into a iso formated date string"""
    return text(Date(adate))


def compactDate(adate):
    """Turns a date in almost any format into a iso formated date string"""
    return Date(adate).compact


def catalanDate(adate):
    """Turns a date in almost any format into a catalan string like '25 de desembre'"""
    return Date(adate).catalanDate


def isoToTime(iso):
    """Turns a string representing an iso time into a time"""
    return datetime.time(*(int(t) for t in iso.split(":")))


class Date(datetime.date):
    """An extension of datetime.date that provides, both
    flexible initialization and flexible formatting via
    special attributes for common representation formats.

    >> "Això va passar el dia {.catalanDate}.".format(date(2013,3,4))
    Això va passar el dia 3 de març de 2013.
    >> "Birth date: {.iso}".format(date.today())
    """

    def __new__(cls, *args):
        """
        Takes as arguments either:
                - an actual datetime.date
                - a tupple (yyyy,mm,dd) or (dd,mm,yyyy)
                - strings in 'yyyy-mm-dd', 'yyyy/mm/dd', 'dd-mm-yyyy', 'dd/mm/yyyy', 'yyyymmdd'
        """
        dateTuple = cls._extractDataTuple(*args)
        return datetime.date.__new__(cls, *dateTuple)

    @classmethod
    def _extractDataTuple(cls, *args):
        if len(args) != 1:
            return cls._extractDataTuple(tuple(args))

        adate = args[0]

        if hasattr(adate, "strftime"):
            return adate.timetuple()[:3]

        # formated string
        for separator in "-/":
            if separator in adate:
                adate = [x for x in adate.split(separator)]

        if len(adate) == 8:
            adate = adate[:4], adate[4:6], adate[6:8]

        if len(adate) != 3:
            raise ValueError("Invalid date initializator '{}'".format(args[0]))

        adate = [int(x) for x in adate]

        # reverse date
        if adate[-1] > 31 and adate[0] < 31:
            adate = adate[::-1]

        return adate

    @property
    def isoDate(self):
        return text(self)

    @property
    def slashDate(self):
        return self.strftime("%d/%m/%Y")

    @property
    def catalanDate(self):
        monthNames = (
            "gener febrer març abril maig juny "
            "juliol agost setembre octubre novembre desembre"
        ).split()
        monthName = monthNames[self.month - 1]
        de = "d'" if monthName[0] in "aeiou" else "de "
        return "{} {}{}".format(self.day, de, monthName)

    @property
    def spanishDate(self):
        monthNames = (
            "enero febrero marzo abril mayo junio "
            "julio agosto septiembre octubre noviembre diciembre"
        ).split()
        monthName = monthNames[self.month - 1]
        return "{} de {}".format(self.day, monthName)

    @property
    def compact(self):
        return self.strftime("%Y%m%d")


class Period:

    def __init__(self, first, lastIncluded):
        """
        Constructs a time period including 'first' day and 'lastIncluded' day,
        being 'first' and 'lastIncluded' strings representing ISO dates YYYY-MM-DD,
        Ie. Period from '2000-01-01' to '2000-01-02' has 2 days.
        When the last day happens before the first day it is considered an empty
        period with zero days.
        """
        self.first = first
        self.last = lastIncluded

    def __iter__(self):
        "Iterates on first and last"
        yield self.first
        yield self.last

    def intersect(self, other):
        """Returns the intersection of the receiver with other.
        If they do not overlap an empty period is returned.
        """
        first = max(self.first, other.first)
        last = min(self.last, other.last)
        return Period(first, last)

    def empty(self):
        """Returns true if the period is empty, that is the
        first day is later than the last day.
        """
        return self.last < self.first

    @property
    def days(self):
        """The number of days of the period"""
        return max((date(self.last) - date(self.first)).days + 1, 0)

    @classmethod
    def trimester(cls, year, trimester):
        """Returns a period of a trimester.
        Payment trimesters start the 10th of january, april, july and october.
        """
        invoicingMonths = ["01", "04", "07", "10"]
        initialDate = "{}-{}-10".format(
            year,
            invoicingMonths[trimester - 1],
        )
        finalDate = "{}-{}-09".format(
            year + 1 if trimester == 4 else year,
            invoicingMonths[trimester % 4],
        )
        return Period(initialDate, finalDate)

    def notUntil(self, start):
        "Returns the period retained until 'start'"
        return Period(max(start, self.first), self.last)

    def notAfter(self, end):
        "Returns the period shortened after 'end'"
        return Period(self.first, min(end, self.last))

    @property
    def t(self):
        """Returns the period as a tuple of iso date strings
        >>> Period("2015-02-20", "2015-03-02").t
        ('2015-02-20', '2015-03-02')
        """
        return tuple(self)


def load_tests(loader, tests, ignore):  # pragma: no cover
    import doctest

    tests.addTests(doctest.DocTestSuite())
    return tests


# vim: sw=4 ts=4 noet



#!/usr/bin/env python3

from .dateutils import (
    Period,
    Date,
    date,
    slashDate,
    isoDate,
    catalanDate,
    isoToTime,
)
import datetime

import unittest

unittest.TestCase.__str__ = unittest.TestCase.id


class DateUtils_Test(unittest.TestCase):

    def test_date_withDate(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date(datetime.date(2015, 10, 30)),
        )

    def test_date_withIso(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date("2015-10-30"),
        )

    def test_date_withTuple(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date((2015, 10, 30)),
        )

    def test_date_withYearMonthDay(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date((2015, 10, 30)),
        )

    def test_date_withTupleReversed(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date(30, 10, 2015),
        )

    def test_date_withSlashDate(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date("2015/10/30"),
        )

    def test_date_withCompact(self):
        self.assertEqual(
            datetime.date(2015, 10, 30),
            date("20151030"),
        )

    def test_date_withBadCompact(self):
        with self.assertRaises(ValueError) as e:
            date("201510303")

        self.assertEqual(e.exception.args[0], "Invalid date initializator '201510303'")

    def test_isoDate_withIso(self):
        self.assertEqual(
            "2015-10-30",
            isoDate("2015-10-30"),
        )

    def test_isoDate_withSlashDate(self):
        self.assertEqual(
            "2015-10-30",
            isoDate("2015/10/30"),
        )

    def test_isoDate_withDate(self):
        self.assertEqual(
            "2015-10-30",
            isoDate(datetime.date(2015, 10, 30)),
        )

    def test_isoDate_withTuple(self):
        self.assertEqual(
            "2015-10-30",
            isoDate((2015, 10, 30)),
        )

    def test_slashDate_withIso(self):
        self.assertEqual(slashDate("2014-09-01"), "01/09/2014")

    def test_slashDate_withDate(self):
        self.assertEqual(slashDate(datetime.date(2014, 9, 1)), "01/09/2014")

    def test_slashDate_withDate(self):
        self.assertEqual(slashDate((2014, 9, 1)), "01/09/2014")

    def test_catalanDate_withTuple(self):
        self.assertEqual(catalanDate((2014, 1, 1)), "1 de gener")

    def test_catalanDate(self):
        self.assertEqual(catalanDate("2014-01-01"), "1 de gener")

    def test_catalanDate_whenMonthStartsWithVowel(self):
        self.assertEqual(catalanDate("2014-08-01"), "1 d'agost")

    def test_Date_catalan(self):
        aDate = Date("2014-01-01")
        self.assertEqual(aDate.catalanDate, "1 de gener")

    def test_Date_spanish(self):
        aDate = Date("2014-01-01")
        self.assertEqual(aDate.spanishDate, "1 de enero")

    def test_Date_format(self):
        aDate = Date("2014-01-02")
        self.assertEqual("hola {} tu".format(aDate), "hola 2014-01-02 tu")

    def test_Date_format_catalan(self):
        aDate = Date("2014-01-02")
        self.assertEqual("hola {.catalanDate} tu".format(aDate), "hola 2 de gener tu")

    def test_Date_format_slash(self):
        aDate = Date("2014-01-02")
        self.assertEqual("hola {.slashDate} tu".format(aDate), "hola 02/01/2014 tu")

    def test_Date_format_compact(self):
        aDate = Date("2014-01-02")
        self.assertEqual(aDate.isoDate, "2014-01-02")

    def test_Date_init_threeArgs(self):
        aDate = Date(2014, 1, 2)
        self.assertEqual("hola {.compact} tu".format(aDate), "hola 20140102 tu")

    def test_isoToTime(self):
        self.assertEqual(isoToTime("01:02:03:0004"), datetime.time(1, 2, 3, 4))

    def test_isoToTime_zero(self):
        self.assertEqual(isoToTime("00:00:00:0000"), datetime.time(0, 0))

    def test_isoToTime_ellided(self):
        self.assertEqual(isoToTime("01:02"), datetime.time(1, 2, 0, 0))

    def test_Date_today(self):
        aDate = Date.today()
        self.assertEqual(aDate.isoDate, "{}".format(datetime.date.today()))


class Period_Test(unittest.TestCase):

    def test_trimesterPeriod_t1(self):
        self.assertEqual(Period.trimester(2015, 1).t, ("2015-01-10", "2015-04-09"))

    def test_trimesterPeriod_t2(self):
        self.assertEqual(Period.trimester(2015, 2).t, ("2015-04-10", "2015-07-09"))

    def test_trimesterPeriod_t3(self):
        self.assertEqual(Period.trimester(2015, 3).t, ("2015-07-10", "2015-10-09"))

    def test_trimesterPeriod_t4(self):
        self.assertEqual(Period.trimester(2015, 4).t, ("2015-10-10", "2016-01-09"))

    def test_empty_whenOrdered(self):
        self.assertFalse(Period("2015-01-10", "2015-01-11").empty())

    def test_empty_whenSame(self):
        self.assertFalse(Period("2015-01-10", "2015-01-10").empty())

    def test_empty_whenInverted(self):
        self.assertTrue(Period("2015-01-10", "2015-01-09").empty())

    def test_notUntil_whenEarly(self):
        p = Period.trimester(2015, 1)
        self.assertEqual(p.notUntil("2014-05-06").t, ("2015-01-10", "2015-04-09"))

    def test_notUntil_whenInside(self):
        p = Period.trimester(2014, 2)
        self.assertEqual(p.notUntil("2014-05-06").t, ("2014-05-06", "2014-07-09"))

    def test_notUntil_whenLater(self):
        p = Period.trimester(2012, 2)
        self.assertEqual(p.notUntil("2014-05-06").t, ("2014-05-06", "2012-07-09"))

    def test_notAfter_whenEarly(self):
        p = Period.trimester(2015, 1)
        self.assertEqual(p.notAfter("2014-05-06").t, ("2015-01-10", "2014-05-06"))

    def test_notAfter_whenInside(self):
        p = Period.trimester(2014, 2)
        self.assertEqual(p.notAfter("2014-05-06").t, ("2014-04-10", "2014-05-06"))

    def test_notAfter_whenLater(self):
        p = Period.trimester(2012, 2)
        self.assertEqual(p.notAfter("2014-05-06").t, ("2012-04-10", "2012-07-09"))

    def test_periodDays(self):
        self.assertEqual(Period("2014-05-06", "2014-05-08").days, 3)

    def test_periodDays_singleDay(self):
        self.assertEqual(Period("2014-05-06", "2014-05-06").days, 1)

    def test_periodDays_leapYear(self):
        self.assertEqual(Period("2012-02-01", "2012-03-01").days, 30)

    def test_periodDays_emptyPeriod(self):
        self.assertEqual(Period("2014-02-01", "2012-02-01").days, 0)

    def test_periodIntersect_whenIncluded(self):
        p1 = Period("2014-01-01", "2014-03-20")
        p2 = Period("2013-01-01", "2015-03-20")
        self.assertEqual(p1.intersect(p2).t, ("2014-01-01", "2014-03-20"))

    def test_periodIntersect_whenIncludes(self):
        p1 = Period("2013-01-01", "2015-03-20")
        p2 = Period("2014-01-01", "2014-03-20")
        self.assertEqual(p1.intersect(p2).t, ("2014-01-01", "2014-03-20"))

    def test_periodIntersect_whenDifers(self):
        p1 = Period("2013-01-01", "2014-03-20")
        p2 = Period("2015-01-01", "2016-03-20")
        self.assertEqual(p1.intersect(p2).t, ("2015-01-01", "2014-03-20"))

    def test_periodIntersect_whenPartiallyOverlap(self):
        p1 = Period("2013-01-01", "2015-03-20")
        p2 = Period("2015-01-01", "2016-03-20")
        self.assertEqual(p1.intersect(p2).t, ("2015-01-01", "2015-03-20"))


# vim: sw=4 ts=4 noet



# Compatibility abstractions

# Discriminating Py2
import sys

py2 = sys.version_info < (3,)

# This resolves pathlib in older Python versions
try:
    from pathlib2 import Path
except ImportError:
    from pathlib import Path


# Ensures that we get an unicode string, even in Py2
def text(data, encoding="utf8"):
    if type(data) is type(""):
        return data
    if type(data) is type(b""):
        return data.decode(encoding)
    return type("")(data)


def isStr(x):
    return type(x) in (
        type(""),
        type(b""),
    )



#!/usr/bin/python3

import unittest
import sys
import io
import os
from .core import _varsTree, _collectVars, namespace
from .nstemplate import apply, extract


class NSTemplate_test(unittest.TestCase):

    def setUp(self):
        self._toRemove = []

    def toRemove(self, f):
        self._toRemove.append(f)

    def tearDown(self):
        for f in self._toRemove:
            if os.path.exists(f):
                os.remove(f)

    def write(self, filename, content):
        with io.open(filename, "w", encoding="utf8") as f:
            f.write(content)

    def read(self, filename):
        with io.open(filename, encoding="utf8") as f:
            return f.read()

    def test_collectVars_withNoVar(self):
        content = "booo"
        result = _collectVars(content)
        self.assertEqual(result, [])

    def test_collectVars_withVar(self):
        content = "b{boo}o"
        result = _collectVars(content)
        self.assertEqual(
            result,
            [
                "boo",
            ],
        )

    def test_collectVars_withManyVarsInALine(self):
        content = "b{boo}o{far}34"
        result = _collectVars(content)
        self.assertEqual(
            result,
            [
                "boo",
                "far",
            ],
        )

    def test_collectVars_multiline(self):
        content = "b{boo}o\ndfs{nice}"
        result = _collectVars(content)
        self.assertEqual(
            result,
            [
                "boo",
                "nice",
            ],
        )

    def test_collectVars_indexinDroped(self):
        content = "dfs{nice[3]}"
        result = _collectVars(content)
        self.assertEqual(
            result,
            [
                "nice",
            ],
        )

    def test_varsTree_plainVars(self):
        theVars = [
            "boo",
            "far",
            "nice",
        ]
        yaml = _varsTree(theVars).dump()
        self.assertEqual(yaml, "boo: ''\n" "far: ''\n" "nice: ''\n")

    def test_varsTree_unordered(self):
        theVars = [
            "nice",
            "far",
            "boo",
        ]
        yaml = _varsTree(theVars).dump()
        self.assertEqual(yaml, "boo: ''\n" "far: ''\n" "nice: ''\n")

    def test_varsTree_withSubVars(self):
        theVars = [
            "upper.boo",
            "upper.far",
            "upper.nice",
        ]
        yaml = _varsTree(theVars).dump()
        self.assertEqual(yaml, "upper:\n" "  boo: ''\n" "  far: ''\n" "  nice: ''\n")

    def test_varsTree_withManySubVars(self):
        theVars = [
            "upper.boo",
            "lower.nice",
            "upper.far",
        ]
        yaml = _varsTree(theVars).dump()
        self.assertEqual(
            yaml, "lower:\n" "  nice: ''\n" "upper:\n" "  boo: ''\n" "  far: ''\n"
        )

    def test_varsTree_withSubSubVars(self):
        theVars = [
            "upper.lower.boo",
            "upper.lower.far",
            "upper.lower.nice",
        ]
        yaml = _varsTree(theVars).dump()
        self.assertEqual(
            yaml,
            "upper:\n" "  lower:\n" "    boo: ''\n" "    far: ''\n" "    nice: ''\n",
        )

    def test_templateVarsAsYaml(self):
        yaml = namespace.fromTemplateVars(
            """\
			{var1.foo} {var1.lala} {var2}
		"""
        )
        self.assertEqual(
            yaml.dump(), "var1:\n" "  foo: ''\n" "  lala: ''\n" "var2: ''\n"
        )

    def test_extract(self):
        filemd = "deleteme.md"
        fileyaml = "deleteme.yaml"
        self.toRemove(filemd)
        self.toRemove(fileyaml)
        self.write(filemd, "{var1} {var2.foo}")

        extract(filemd, fileyaml)

        self.assertEqual(self.read(fileyaml), "var1: ''\n" "var2:\n" "  foo: ''\n")

    def test_apply(self):
        filemd = "deleteme.md"
        fileyaml = "deleteme.yaml"
        fileout = "deleteme-output.md"
        self.toRemove(filemd)
        self.toRemove(fileyaml)
        self.toRemove(fileout)

        self.write(filemd, "{var1} {var2.foo}")
        self.write(fileyaml, "var1: 'value1'\n" "var2:\n" "  foo: 666\n")

        apply(fileyaml, filemd, fileout)

        self.assertEqual(self.read(fileout), "value1 666")


# vim: sw=4 ts=4 noet



#!/usr/bin/python3
# -*- coding: utf-8 -*-

from .core import namespace
from .compat import Path, py2
from .dateutils import Date
import decimal
import datetime
import math
import unittest
import io
import sys


class Namespace_Test(unittest.TestCase):

    def setUp(self):
        """"""

    def tearDown(self):
        import os

        try:
            os.unlink("test.yaml")
        except:
            pass

    def assertContent(self, expected):
        import io

        with io.open("test.yaml", encoding="utf-8") as f:
            result = f.read()
        self.assertEqual(result, expected)

    def writeFile(self, data):
        import io

        with io.open("test.yaml", "w", encoding="utf-8") as f:
            f.write(data)

    def test_construction_default(self):
        ns = namespace()
        self.assertEqual(ns.dump(), "{}\n")

    def test_setattribute(self):
        ns = namespace()
        ns.lala = "boo"
        self.assertEqual(ns.lala, "boo")
        self.assertEqual(ns["lala"], "boo")
        self.assertEqual(ns.dump(), "lala: boo\n")

    def test_setattribute_twice_overwrites(self):
        ns = namespace()
        ns.lala = "boo"
        ns.lala = "foo"
        self.assertEqual(ns.lala, "foo")
        self.assertEqual(ns["lala"], "foo")
        self.assertEqual(ns.dump(), "lala: foo\n")

    def test_setattribute_many(self):
        ns = namespace()
        ns.lala = "boo"
        ns.lola = "foo"
        self.assertEqual(ns.lola, "foo")
        self.assertEqual(ns["lola"], "foo")
        self.assertEqual(ns.dump(), "lala: boo\n" "lola: foo\n")

    def test_setattribute_differentOrder(self):
        ns = namespace()
        ns.lola = "foo"
        ns.lala = "boo"
        self.assertEqual(ns.dump(), "lola: foo\n" "lala: boo\n")

    def test_getattribute_missing(self):
        ns = namespace()
        with self.assertRaises(AttributeError) as ctx:
            ns.nonexisting
        self.assertEqual(format(ctx.exception), "nonexisting")

    def test_delattribute(self):
        ns = namespace()
        ns.lola = "foo"
        ns.lala = "boo"
        del ns.lola
        self.assertEqual(ns.dump(), "lala: boo\n")

    def test_delattribute(self):
        ns = namespace()
        ns.lola = "foo"
        with self.assertRaises(AttributeError):
            del ns.notexisting

    def test_load_None(self):
        self.assertEqual(namespace.loads(""), None)

    def test_load_empty(self):
        self.assertEqual(namespace.loads("{}"), namespace())

    def test_reload(self):
        yamlcontent = "lala: boo\n" "lola: foo\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(ns.dump(), yamlcontent)

    def test_reload_inverseOrder(self):
        yamlcontent = "lola: foo\n" "lala: boo\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(ns.dump(), yamlcontent)

    def test_subnamespaces(self):
        ns = namespace()
        ns.sub = namespace()
        ns.sub.foo = "value1"
        ns.sub.bar = "value2"
        self.assertEqual(ns.dump(), "sub:\n" "  foo: value1\n" "  bar: value2\n")

    def test_reload_subnamespaces(self):
        yamlcontent = "sub:\n" "  foo: value1\n" "  bar: value2\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(ns.dump(), yamlcontent)

    def test_loads_fromEncodedUtf8(self):
        yaml = "hi: caña\n".encode("utf-8")
        result = namespace.loads(yaml)
        self.assertEqual(result, namespace(hi="caña"))

    def test_loads_unicode(self):
        yaml = "hi: caña\n"
        result = namespace.loads(yaml)
        self.assertEqual(result, namespace(hi="caña"))

    # Numbers (int, decimal, float)

    def test_load_sexagesimal(self):
        ns = namespace.loads("attribute: 120:20:04")
        self.assertEqual(type(ns.attribute), int)
        self.assertEqual(ns.attribute, (120 * 60 + 20) * 60 + 4.0)

    def test_load_decimal(self):
        yamlcontent = "attribute: 3.41"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.attribute), decimal.Decimal)
        self.assertEqual(ns.attribute, decimal.Decimal("3.41"))

    def test_load_decimal_negative(self):
        yamlcontent = "attribute: -3.41"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.attribute), decimal.Decimal)
        self.assertEqual(ns.attribute, decimal.Decimal("-3.41"))

    def test_load_decimal_infinite(self):
        ns = namespace.loads("attribute: .inf")
        self.assertEqual(type(ns.attribute), decimal.Decimal)
        self.assertEqual(ns.attribute, decimal.Decimal("Infinity"))

    def test_load_decimal_minusInfinite(self):
        ns = namespace.loads("attribute: -.inf")
        self.assertEqual(type(ns.attribute), decimal.Decimal)
        self.assertEqual(ns.attribute, decimal.Decimal("-Infinity"))

    def test_load_decimal_notANumber(self):
        ns = namespace.loads("attribute: .nan")
        self.assertEqual(type(ns.attribute), decimal.Decimal)
        self.assertTrue(ns.attribute.is_nan())

    def test_dump_decimal_inf(self):
        ns = namespace()
        ns.attribute = decimal.Decimal("inf")
        self.assertEqual(ns.dump(), "attribute: .inf\n")

    def test_dump_decimal_minus_inf(self):
        ns = namespace()
        ns.attribute = decimal.Decimal("-inf")
        self.assertEqual(ns.dump(), "attribute: -.inf\n")

    def test_dump_float(self):
        ns = namespace()
        ns.attribute = 3.41
        self.assertEqual(ns.dump(), "attribute: 3.41\n")

    def test_dump_float_inf(self):

        ns = namespace()
        ns.attribute = float("inf")
        self.assertEqual(ns.dump(), "attribute: .inf\n")

    def test_dump_float_minusInf(self):

        ns = namespace()
        ns.attribute = float("-inf")
        self.assertEqual(ns.dump(), "attribute: -.inf\n")

    def test_dump_decimal(self):
        ns = namespace()
        ns.attribute = decimal.Decimal("3.41")
        self.assertEqual(ns.dump(), "attribute: 3.41\n")

    def test_dump_float_nan(self):
        ns = namespace()
        ns.attribute = float("nan")
        self.assertEqual(ns.dump(), "attribute: .nan\n")

    def test_dump_decimal_nan(self):
        ns = namespace()
        ns.attribute = decimal.Decimal("NaN")
        self.assertEqual(ns.dump(), "attribute: .nan\n")

    def test_dump_cientifical_notation(self):
        ns = namespace()
        ns.attribute = 1e17
        self.assertEqual(ns.dump(), "attribute: 1.0e+17\n")

    # Dates, times

    def test_load_date(self):
        yamlcontent = "adate: 2000-02-28\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.adate), Date)

    def test_dump_dateutilsdate(self):
        yamlcontent = "adate: 2000-02-28\n"
        ns = namespace()
        ns.adate = Date(2000, 2, 28)
        self.assertEqual(ns.dump(), yamlcontent)

    def test_dump_date(self):
        yamlcontent = "adate: 2000-02-28\n"
        ns = namespace()
        ns.adate = datetime.date(2000, 2, 28)
        self.assertEqual(ns.dump(), yamlcontent)

    class tzoffset(datetime.tzinfo):
        def __init__(self, name=None, *args, **kwds):
            self._offset = datetime.timedelta(*args, **kwds)
            self._name = name

        def utcoffset(self, dt):
            return self._offset

        def dst(self):
            return datetime.timedelta(0)

        def tzname(self):
            return self._name

    def test_dump_datetime(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040+02:00\n"
        ns = namespace()
        ns.adate = datetime.datetime(
            2000, 2, 28, 10, 20, 30, 40, self.tzoffset(hours=2)
        )
        self.assertEqual(ns.dump(), yamlcontent)

    def test_dump_datetime_utc(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040+00:00\n"
        ns = namespace()
        ns.adate = datetime.datetime(
            2000, 2, 28, 10, 20, 30, 40, self.tzoffset(hours=0)
        )
        self.assertEqual(ns.dump(), yamlcontent)

    def test_dump_datetime_naive(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040\n"
        ns = namespace()
        ns.adate = datetime.datetime(2000, 2, 28, 10, 20, 30, 40)
        self.assertEqual(ns.dump(), yamlcontent)

    def test_load_datetime(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040+02:00\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.adate), datetime.datetime)
        self.assertEqual(
            ns.adate,
            datetime.datetime(2000, 2, 28, 10, 20, 30, 40, self.tzoffset(hours=2)),
        )

    def test_load_datetime_utc(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040Z\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.adate), datetime.datetime)
        self.assertEqual(
            ns.adate,
            datetime.datetime(2000, 2, 28, 10, 20, 30, 40, self.tzoffset(hours=0)),
        )

    def test_load_datetime_naive(self):
        yamlcontent = "adate: 2000-02-28 10:20:30.000040\n"
        ns = namespace.loads(yamlcontent)
        self.assertEqual(type(ns.adate), datetime.datetime)
        self.assertEqual(ns.adate, datetime.datetime(2000, 2, 28, 10, 20, 30, 40))

    # dir just for values accessible as attribute

    def test_dir_withNonExistingAttributes(self):
        ns = namespace()
        ns.attr1 = "value1"
        self.assertTrue("bad" not in dir(ns))

    def test_dir_withExistingAttributes(self):
        ns = namespace()
        ns.attr1 = "value1"
        self.assertTrue("attr1" in dir(ns))

    def test_dir_withNonIdentifierNames(self):
        ns = namespace()
        ns["a-b"] = "value1"

        self.assertTrue("a-b" not in dir(ns))

    # multiline strings

    def test_dump_multiline(self):
        ns = namespace()
        ns.text = "line\nsecond line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), "text: |-\n" "  line\n" "  second line\n" "other: 2\n"
        )

    def test_dump_multiline_innnerIndentation(self):
        ns = namespace()
        ns.text = "line\n  second line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), "text: |-\n" "  line\n" "    second line\n" "other: 2\n"
        )

    def test_dump_multiline_firstLineIndentation(self):
        ns = namespace()
        ns.text = "  line\nsecond line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), "text: |2-\n" "    line\n" "  second line\n" "other: 2\n"
        )

    def test_dump_multiline_tabIndentation(self):
        ns = namespace()
        ns.text = "\tline\nsecond line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), 'text: "\\tline\\nsecond line"\n' "other: 2\n"
        )

    def test_dump_multiline_endSpaces(self):
        ns = namespace()
        ns.text = "line   \nsecond line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), 'text: "line   \\nsecond line"\n' "other: 2\n"
        )

    def test_dump_multiline_emptyLine(self):
        ns = namespace()
        ns.text = "line\n\nsecond line"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(), "text: |-\n" "  line\n" "\n" "  second line\n" "other: 2\n"
        )

    def test_dump_multiline_emptyAtEnd(self):
        ns = namespace()
        ns.text = "line\nsecond line\n"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(),
            "text: |\n"  # this changes, was |-
            "  line\n"
            "  second line\n"
            "other: 2\n",
        )

    def test_dump_multiline_manyEmptyAtEnd(self):
        ns = namespace()
        ns.text = "line\nsecond line\n\n\n"
        ns.other = 2
        self.assertMultiLineEqual(
            ns.dump(),
            "text: |+\n"  # this changes, was |-
            "  line\n"
            "  second line\n"
            "\n"
            "\n"
            "other: 2\n"
            "...\n",  # Why dumper adds this?
        )

    # Other

    def test_load_recursiveArray(self):
        ns = namespace.loads("list:\n" "  - key1: value1\n")
        self.assertEqual(ns, namespace(list=[namespace(key1="value1")]))

    # Sources an targets

    def test_load_fromFilename(self):
        self.writeFile("hi: caña\n")
        result = namespace.load("test.yaml")
        self.assertEqual(result, namespace(hi="caña"))

    def test_dump_toFileName(self):
        data = namespace(otra="caña")
        data.dump("test.yaml")
        self.assertContent("otra: caña\n")

    @unittest.skipIf(not Path, "neither pathlib or pathlib2 not installed")
    def test_load_fromPath(self):
        self.writeFile("hi: caña\n")
        result = namespace.load(Path("test.yaml"))
        self.assertEqual(result, namespace(hi="caña"))

    @unittest.skipIf(not Path, "neither pathlib or pathlib2 not installed")
    def test_dump_toPath(self):
        data = namespace(otra="caña")
        data.dump(Path("test.yaml"))
        self.assertContent("otra: caña\n")

    @unittest.skipIf(not py2, "PyYaml in Python 3 requires a text file")
    def test_load_fromBinaryFileObject(self):
        self.writeFile("hi: caña\n")
        with io.open("test.yaml", "rb") as f:
            result = namespace.load(f)
        self.assertEqual(result, namespace(hi="caña"))

    @unittest.skipIf(not py2, "PyYaml in Python 3 requires a text file")
    def test_dump_toBinaryFileObject(self):
        data = namespace(otra="caña")
        with io.open("test.yaml", "wb") as f:
            data.dump(f)
        self.assertContent("otra: caña\n")

    @unittest.skipIf(py2, "PyYaml in Python 2 requires a binary file")
    def test_load_fromTextFileObject(self):
        self.writeFile("hi: caña\n")
        with io.open("test.yaml", "r", encoding="utf8") as f:
            result = namespace.load(f)
        self.assertEqual(result, namespace(hi="caña"))

    @unittest.skipIf(py2, "PyYaml in Python 2 requires a binary file")
    def test_dump_toTextFileObject(self):
        data = namespace(otra="caña")
        with io.open("test.yaml", "w", encoding="utf8") as f:
            data.dump(f)
        self.assertContent("otra: caña\n")

    def test_deep_int(self):
        self.assertEqual(namespace.deep(2), 2)

    def test_deep_dict(self):
        result = namespace.deep(dict(a=2))
        self.assertEqual(
            result,
            namespace(a=2),
        )
        self.assertEqual(type(result), namespace)

    def test_deep_subdict(self):
        result = namespace.deep(dict(a=dict(b=2)))
        self.assertEqual(
            result,
            namespace(a=namespace(b=2)),
        )
        self.assertEqual(type(result), namespace)
        self.assertEqual(type(result.a), namespace)

    def test_deep_ns_keeporder(self):
        result = namespace.deep(
            namespace(
                [
                    ("last", 3),
                    ("first", "value"),
                ]
            )
        )
        self.assertEqual(
            [k for k, v in result.items()],
            ["last", "first"],
        )
        self.assertEqual(type(result), namespace)

    def test_deep_ns_sorted(self):
        result = namespace.deep(
            namespace(
                [
                    ("last", 3),
                    ("first", "value"),
                ]
            ),
            sorted=True,
        )
        self.assertEqual(
            [k for k, v in result.items()],
            ["first", "last"],
        )
        self.assertEqual(type(result), namespace)

    def test_deep_ns_subdict(self):
        result = namespace.deep(namespace(a=dict(b=2)))
        self.assertEqual(
            result,
            namespace(a=namespace(b=2)),
        )
        self.assertEqual(type(result), namespace)
        self.assertEqual(type(result.a), namespace)

    def test_deep_list_ns(self):
        result = namespace.deep([3, dict(a=55)])
        self.assertEqual(type(result), list)
        self.assertEqual(type(result[1]), namespace)

    def test_deep_tuple_ns(self):
        result = namespace.deep((3, dict(a=55)))
        self.assertEqual(type(result), list)
        self.assertEqual(type(result[1]), namespace)

    def test_deep_list_ns_sorted(self):
        result = namespace.deep(
            [
                namespace(
                    [
                        ("last", 3),
                        ("first", "value"),
                    ]
                ),
            ],
            sorted=True,
        )
        self.assertEqual(
            [k for k, v in result[0].items()],
            ["first", "last"],
        )

    def test_deep_list_ns_keepOrder(self):
        result = namespace.deep(
            [
                namespace(
                    [
                        ("last", 3),
                        ("first", "value"),
                    ]
                ),
            ],
        )
        self.assertEqual(
            [k for k, v in result[0].items()],
            ["last", "first"],
        )

    def test_deep_ns_ns_sorted(self):
        result = namespace.deep(
            namespace(
                a=namespace(
                    [
                        ("last", 3),
                        ("first", "value"),
                    ]
                ),
            ),
            sorted=True,
        )
        self.assertEqual(
            [k for k, v in result.a.items()],
            ["first", "last"],
        )

    def test_deep_ns_ns_keepOrder(self):
        result = namespace.deep(
            namespace(
                a=namespace(
                    [
                        ("last", 3),
                        ("first", "value"),
                    ]
                ),
            ),
        )
        self.assertEqual(
            [k for k, v in result.a.items()],
            ["last", "first"],
        )

    def test_getitem_multilevel(self):
        data = namespace(a=namespace(b=4))
        self.assertEqual(data["a.b"], 4)

    def test_setitem_multilevel(self):
        data = namespace()
        data["a.b"] = 4
        self.assertEqual(data, namespace(a=namespace(b=4)))

    def test_non_string_key(self):
        data = namespace()
        data[3, "key"] = "value"
        self.assertEqual(data[3, "key"], "value")
        namespace.loads("['key', 3]: value")


# vim: sw=4 ts=4 noet



import decimal
import datetime
from .core import namespace
from .dateutils import Date
from .compat import text

try:
    import numpy as np
except ImportError:
    np = None

import yaml

try:
    from yaml import CSafeLoader as SafeLoader, CSafeDumper as SafeDumper
except ImportError:
    from yaml import SafeLoader, SafeDumper


class NamespaceYamlDumper(SafeDumper):

    def __init__(self, *args, **kwargs):
        super(NamespaceYamlDumper, self).__init__(*args, **kwargs)
        self.add_representer(namespace, NamespaceYamlDumper.represent_dict)
        self.add_representer(decimal.Decimal, NamespaceYamlDumper.represent_float)
        self.add_representer(Date, NamespaceYamlDumper.represent_date)
        if np:
            self.add_representer(np.ndarray, NamespaceYamlDumper.represent_np)

        self.add_representer(type(""), NamespaceYamlDumper.represent_str)
        if type("") != type(""):  # Py2 compat
            self.add_representer(type(""), NamespaceYamlDumper.represent_str)

    def represent_str(self, data):
        if "\n" in data:  # check for multiline string
            return self.represent_scalar("tag:yaml.org,2002:str", data, style="|")
        return self.represent_scalar("tag:yaml.org,2002:str", data)

    def represent_date(self, data):
        return self.represent_scalar("tag:yaml.org,2002:timestamp", type("")(data))

    def represent_float(self, data):
        if data != data or (data == 0.0 and data == 1.0):
            value = ".nan"
        elif data == self.inf_value:
            value = ".inf"
        elif data == -self.inf_value:
            value = "-.inf"
        else:
            value = text(data).lower()

        return self.represent_scalar("tag:yaml.org,2002:float", value)

    def represent_np(self, data):
        return self.represent_sequence(
            "tag:yaml.org,2002:seq", [float(x) for x in data]
        )

    # Kludge: This is a rewritten version of yaml.representer.Representer.represent_mapping
    # to avoid sorting pairs by key
    def represent_mapping(self, tag, mapping, flow_style=None):
        value = []
        node = yaml.nodes.MappingNode(tag, value, flow_style=flow_style)
        if self.alias_key is not None:
            self.represented_objects[self.alias_key] = node
        best_style = True
        if hasattr(mapping, "items"):
            mapping = list(mapping.items())
        # Those are the lines removed from official sources
        # 		try:
        # 			mapping = sorted(mapping)
        # 		except TypeError:
        # 			pass
        for item_key, item_value in mapping:
            node_key = self.represent_data(item_key)
            node_value = self.represent_data(item_value)
            if not (isinstance(node_key, yaml.nodes.ScalarNode) and not node_key.style):
                best_style = False
            if not (
                isinstance(node_value, yaml.nodes.ScalarNode) and not node_value.style
            ):
                best_style = False
            value.append((node_key, node_value))
        if flow_style is None:
            if self.default_flow_style is not None:
                node.flow_style = self.default_flow_style
            else:
                node.flow_style = best_style
        return node


class NamespaceYAMLLoader(SafeLoader):

    def __init__(self, *args, **kwargs):
        super(NamespaceYAMLLoader, self).__init__(*args, **kwargs)
        self.add_constructor("tag:yaml.org,2002:map", type(self).construct_yaml_map)
        self.add_constructor("tag:yaml.org,2002:omap", type(self).construct_yaml_map)
        self.add_constructor("tag:yaml.org,2002:float", type(self).construct_decimal)
        self.add_constructor(
            "tag:yaml.org,2002:timestamp", type(self).construct_yaml_timestamp
        )

    def construct_decimal(self, node):
        value = self.construct_scalar(node)
        value = value.replace("_", "").lower()
        sign = +1
        if value[0] == "-":
            sign = -1
        if value[0] in "+-":
            value = value[1:]
        if value == ".inf":
            return sign * decimal.Decimal("Infinity")
        if value == ".nan":
            return decimal.Decimal("NaN")
        return sign * decimal.Decimal(value)

    def construct_yaml_map(self, node):
        data = namespace()
        yield data
        value = self.construct_mapping(node)
        data.update(value)

    def construct_yaml_timestamp(self, node):
        result = super(NamespaceYAMLLoader, self).construct_yaml_timestamp(node)
        if not isinstance(result, datetime.datetime):
            return Date(result)
        return result

    def construct_mapping(self, node, deep=False):
        if isinstance(node, yaml.MappingNode):
            self.flatten_mapping(node)
        else:
            raise yaml.constructor.ConstructorError(
                None,
                None,
                "expected a mapping node, but found {}".format(node.id),
                node.start_mark,
            )

        mapping = namespace()
        for key_node, value_node in node.value:
            key = self.construct_object(
                key_node, deep=True
            )  # default is to not recurse into keys
            if isinstance(key, list):
                key = tuple(key)
            try:
                hash(key)
            except TypeError as exc:
                raise yaml.constructor.ConstructorError(
                    "while constructing a mapping",
                    node.start_mark,
                    "found unacceptable key ({})".format(exc),
                    key_node.start_mark,
                )
            value = self.construct_object(value_node, deep=deep)
            mapping[key] = value
        return mapping



#!/usr/bin/env python
from setuptools import setup, find_packages
import sys

readme = open("README.md").read()
py2 = sys.version_info < (3,)

setup(
	name = "yamlns",
	version = "0.12.3",
	description = "YAML serializable dictionary with extra attribute accessors and more",
	author = "David Garcia Garzon",
	author_email = "voki@canvoki.net",
	url = 'https://github.com/GuifiBaix/python-yamlns',
	long_description = readme,
	long_description_content_type = 'text/markdown',
	license = 'GNU General Public License v3 or later (GPLv3+)',
	packages=find_packages(exclude=['*_[tT]est*']),
	entry_points=dict(
		console_scripts = [
			'json2yaml = yamlns.cli.json2yaml:main',
			'nstemplate = yamlns.cli.nstemplate:main',
			'nstemplate.py = yamlns.cli.nstemplate:deprecated', # backwards compatibility
		],
	),
	install_requires=[
		'setuptools>=20.4', # markdown readme
		'PyYAML<6, >5.3.1' if py2 else 'PyYAML>=5.3.1', # security
		'pytest<4.7' if py2 else 'pytest',
		'pytest-cov',
		'pathlib2' if py2 else '', # Py2 backport
		'packaging<21' if py2 else '', # Py2, indirect importlib-metadata, zipp
		'zipp<2' if py2 else '', # Py2, indirect importlib-metadata
		'configparser<5' if py2 else '', # Py2, indirect importlib-metadata
		'importlib-metadata<3' if py2 else '', # Py2, indirect pytest
		'click<8' if py2 else '', # Py2, indirect pytest
		'coverage<6' if py2 else '', # Py2, indirect pytest-cov
		'pyparsing<3' if py2 else '', # Py2, indirect packaging
		'contextlib2<21' if py2 else '', # Py2, indirect by many libs
		'attrs<22' if py2 else '', # Py2, indirect by many libs
	],
	include_package_data = True,
	test_suite = 'yamlns',
	classifiers = [
		'Programming Language :: Python',
		'Programming Language :: Python :: 2',
		'Programming Language :: Python :: 3',
		'Topic :: Software Development :: Libraries :: Python Modules',
		'Framework :: Pytest',
		'Intended Audience :: Developers',
		'Development Status :: 5 - Production/Stable',
		'License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)',
		'Operating System :: OS Independent',
	],
)

# vim: sw=4 ts=4 noet



import sys
from ..core import ns


def main():  # pragma: no cover
    import argparse

    parser = argparse.ArgumentParser(
        description="Converts JSON into pretty printed YAML.",
    )
    args = parser.parse_args()

    data = ns.load(sys.stdin)
    data.dump(sys.stdout)



# DO NOT IMPORT THIS FILE DIRECTLY FROM pytestutils
# IT WOULD BREAK pytest ASSERT REWRITE
from ..compat import py2


def _ensure_pytest_assert_rewrite_active():  # pragma: no cover
    spec = globals().get("__spec__")
    loader = getattr(spec, "loader", None)

    if loader and loader.__class__.__name__ == "AssertionRewritingHook":
        # Using rewrite loader -> good
        return

    msg = [
        "assert_ns_equal was imported without pytest assert rewriting.",
        "",
        "Import it from yamlns.pytestutils",
        "",
    ]

    raise RuntimeError("\n".join(msg))


if not py2:
    _ensure_pytest_assert_rewrite_active()


from ..testutils import (
    _parse_normalize_and_dump,
)


def assert_ns_equal(data, expectation):
    """
    Assert that data representation in yaml matches the expectation.
    Both ends can be either a dictionary like object or a yaml string.
    """
    assert _parse_normalize_and_dump(data) == _parse_normalize_and_dump(expectation)



# -*- encoding: utf8 -*-

import unittest
from .core import ns


class TestUtils_Test(unittest.TestCase):

    from .testutils import assertNsEqual
    from .testutils import assertNsContains

    def test_assertNsEqual_differentValues(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual(ns(a=3), ns(a=4))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\n' != 'a: 4\\n'\n" "- a: 3\n" "?    ^\n" "+ a: 4\n" "?    ^\n",
        )

    def test_assertNsEqual_differentKeys(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual(ns(b=2), ns(c=2))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'b: 2\\n' != 'c: 2\\n'\n" "- b: 2\n" "? ^\n" "+ c: 2\n" "? ^\n",
        )

    def test_assertNsEqual_withOneDict(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual(dict(a=3), ns(a=4))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\n' != 'a: 4\\n'\n" "- a: 3\n" "?    ^\n" "+ a: 4\n" "?    ^\n",
        )

    def test_assertNsEqual_withOtherDict(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual(ns(a=3), dict(a=4))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\n' != 'a: 4\\n'\n" "- a: 3\n" "?    ^\n" "+ a: 4\n" "?    ^\n",
        )

    def test_assertNsEqual_withOneYaml(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual("a: 3", ns(a=4))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\n' != 'a: 4\\n'\n" "- a: 3\n" "?    ^\n" "+ a: 4\n" "?    ^\n",
        )

    def test_assertNsEqual_withOtheYaml(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual(ns(a=3), "a: 4")
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\n' != 'a: 4\\n'\n" "- a: 3\n" "?    ^\n" "+ a: 4\n" "?    ^\n",
        )

    def test_assertNsEqual_keysGetSorted(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual("a: 3\nb: 4", "b: 4\na: different")
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a: 3\\nb: 4\\n' != 'a: different\\nb: 4\\n'\n"
            "- a: 3\n"
            "+ a: different\n"
            "  b: 4\n",
        )

    def test_assertNsEqual_innerDictsGetSortedToo(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual("a:\n  b: 4\n  c: 3", "a:\n  c: 3\n  b: different")
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a:\\n  b: 4\\n  c: 3\\n' != 'a:\\n  b: different\\n  c: 3\\n'\n"
            "  a:\n"
            "-   b: 4\n"
            "+   b: different\n"
            "    c: 3\n",
        )

    def test_assertNsEqual_dictInsideList(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual("a: [{c: 3, b: 2}, 4]", "a: [{b: 2}, 4]")
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'a:\\n- b: 2\\n  c: 3\\n- 4\\n' != 'a:\\n- b: 2\\n- 4\\n'\n"
            "  a:\n"
            "  - b: 2\n"
            "-   c: 3\n"
            "  - 4\n",
        )

    def test_assertNsEqual_lists(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsEqual("[3, 4]", [3, 5])
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            "'data:\\n- 3\\n- 4\\n' != 'data:\\n- 3\\n- 5\\n'\n"
            "  data:\n"
            "  - 3\n"
            "- - 4\n"
            "?   ^\n"
            "+ - 5\n"
            "?   ^\n",
        )

    def test__assertNsContains__equals(self):
        self.assertNsContains(ns(akey="value"), ns(akey="value"))
        # Should not raise

    def test__assertNsContains__differingKey(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsContains(ns(common="different"), ns(common="value"))
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            """'common: different\\n' != 'common: value\\n'\n"""
            """- common: different\n"""
            """+ common: value\n""",
        )

    def test__assertNsContains__missingKey(self):
        with self.assertRaises(AssertionError) as ctx:
            self.assertNsContains(
                ns(common="value"), ns(common="value", missing="value")
            )
        self.assertMultiLineEqual(
            type("u")(ctx.exception),
            """'common: value\\n' != 'common: value\\nmissing: value\\n'\n"""
            """  common: value\n"""
            """+ missing: value\n""",
        )

    def test__assertNsContains__ignoredKey(self):
        self.assertNsContains(ns(common="value", ignored="value"), ns(common="value"))
        # Should not raise


# vim: noet ts=4 sw=4



# -*- coding: utf-8 -*-

import unittest
from .core import ns
from .compat import isStr

# Readable verbose testcase listing
unittest.TestCase.__str__ = unittest.TestCase.id


def assertNsEqual(self, dict1, dict2):
    """
    Asserts that both dicts have equivalent structure
    by comparing its dump as YAML with all the keys
    alphabetically sorted
    If parameters are strings they are parsed as yaml.

    YAML dump of both structures are
    Comparation by comparing the result of turning them
    to yaml sorting the keys of any dict within the structure.
    """

    def parseIfString(x):
        if isStr(x):
            return ns.loads(x)
        return x

    def yaml(x):
        x = parseIfString(x)
        if type(x) in (list, tuple):
            return normalize(ns(data=x)).dump()
        return normalize(x).dump()

    self.assertMultiLineEqual(
        yaml(dict1),
        yaml(dict2),
    )


def normalize(x):
    """Turns recursively all the dicts of a json like
    structure into yamlns namespaces with their keys sorted
    so that their dumps can be compared.
    """
    return ns.deep(x, sorted=True)


def _parse_and_normalize(x):
    if isStr(x):
        x = ns.loads(x)
    return normalize(x)


def _parse_normalize_and_dump(x):
    x = _parse_and_normalize(x)
    if type(x) == ns:
        return x.dump()
    return x


def assertNsContains(self, data, expected):
    """
    Assert that all keys in expected have the same values
    in data than in expected.
    """

    def filter_keys(x, reference):
        if not isinstance(x, dict):
            return x
        return ns(
            ((k, filter_keys(v, reference[k])) for k, v in x.items() if k in reference)
        )

    data = _parse_and_normalize(data)
    expected = _parse_and_normalize(expected)
    self.assertNsEqual(filter_keys(data, expected).dump(), expected.dump())


# vim: ts=4 sw=4 et



from .core import ns
from .compat import Path, text
from ._pytestutils_asserts.assert_impl import assert_ns_equal
import pytest
from .testutils import (
    normalize,
)


@pytest.fixture
def test_name(request):
    """Returns the test name"""
    return "{}.{}".format(request.node.module.__name__, request.node.name)


@pytest.fixture
def text_snapshot(test_name):
    """Returns an assertion function that compares the value with the last accepted execution.
    former results are stored in 'testdata/snapshots' with the name of the test and '.expected'
    suffix.
    The first time you run it or every time the tests fails a file with suffix '.result'
    will be generated.
    """

    def assertion(result, name=None):
        snapshotdir = Path("testdata/snapshots/")
        name = name or test_name
        snapshotfile = snapshotdir / (name + ".expected")
        resultfile = snapshotdir / (name + ".result")
        snapshotdir.mkdir(parents=True, exist_ok=True)
        resultfile.write_text(text(result), encoding="utf8")
        assert (
            snapshotfile.exists()
        ), "First snapshot, check results and accept them with:\n" "mv {} {}\n".format(
            resultfile, snapshotfile
        )
        assert resultfile.read_text(encoding="utf8") == snapshotfile.read_text(
            "utf8"
        ), (
            "Failed snapshot. Check the result and if it is ok accept it with:\n"
            "mv {} {}\n".format(resultfile, snapshotfile)
        )
        # the result is keept if any of the former asserts fails
        resultfile.unlink()

    return assertion


@pytest.fixture
def yaml_snapshot(text_snapshot):
    """Returns an assertion function that compares the value with the last accepted execution.
    former results are stored in 'testdata/snapshots' with the name of the test and '.expected'
    suffix.
    The first time you run it or every time the tests fails a file with suffix '.result'
    will be generated.
    """

    def assertion(data, name=None):
        text = ns(snapshot=normalize(data)).dump()
        text_snapshot(text, name)

    return assertion


def assert_ns_contains(data, expected):
    """
    Assert that all keys in expected have the same values
    in data than in expected.
    """

    def filter_keys(x, reference):
        if not isinstance(x, dict):
            return x
        return ns(
            ((k, filter_keys(v, reference[k])) for k, v in x.items() if k in reference)
        )

    assert_ns_equal(filter_keys(data, expected), expected)
